@@ -14,14 +14,14 @@ use crate::signature::verify_signed_content;
 
 async fn handle_delete_internal(
 	garage: &Garage,
-	bucket: &str,
+	bucket_id: Uuid,
 	key: &str,
 ) -> Result<(Uuid, Uuid), Error> {
 	let object = garage
 		.object_table
-		.get(&bucket.to_string(), &key.to_string())
+		.get(&bucket_id, &key.to_string())
 		.await?
-		.ok_or(Error::NotFound)?; // No need to delete
+		.ok_or(Error::NoSuchKey)?; // No need to delete
 
 	let interesting_versions = object.versions().iter().filter(|v| {
 		!matches!(
@@ -40,12 +40,12 @@ async fn handle_delete_internal(
 		timestamp = std::cmp::max(timestamp, v.timestamp + 1);
 	}
 
-	let deleted_version = version_to_delete.ok_or(Error::NotFound)?;
+	let deleted_version = version_to_delete.ok_or(Error::NoSuchKey)?;
 
 	let version_uuid = gen_uuid();
 
 	let object = Object::new(
-		bucket.into(),
+		bucket_id,
 		key.into(),
 		vec![ObjectVersion {
 			uuid: version_uuid,
@@ -61,11 +61,11 @@ async fn handle_delete_internal(
 
 pub async fn handle_delete(
 	garage: Arc<Garage>,
-	bucket: &str,
+	bucket_id: Uuid,
 	key: &str,
 ) -> Result<Response<Body>, Error> {
 	let (_deleted_version, delete_marker_version) =
-		handle_delete_internal(&garage, bucket, key).await?;
+		handle_delete_internal(&garage, bucket_id, key).await?;
 
 	Ok(Response::builder()
 		.header("x-amz-version-id", hex::encode(delete_marker_version))
@@ -76,12 +76,15 @@ pub async fn handle_delete(
 
 pub async fn handle_delete_objects(
 	garage: Arc<Garage>,
-	bucket: &str,
+	bucket_id: Uuid,
 	req: Request<Body>,
 	content_sha256: Option<Hash>,
 ) -> Result<Response<Body>, Error> {
 	let body = hyper::body::to_bytes(req.into_body()).await?;
-	verify_signed_content(content_sha256, &body[..])?;
+
+	if let Some(content_sha256) = content_sha256 {
+		verify_signed_content(content_sha256, &body[..])?;
+	}
 
 	let cmd_xml = roxmltree::Document::parse(std::str::from_utf8(&body)?)?;
 	let cmd = parse_delete_objects_xml(&cmd_xml).ok_or_bad_request("Invalid delete XML query")?;
@@ -90,7 +93,7 @@ pub async fn handle_delete_objects(
 	let mut ret_errors = Vec::new();
 
 	for obj in cmd.objects.iter() {
-		match handle_delete_internal(&garage, bucket, &obj.key).await {
+		match handle_delete_internal(&garage, bucket_id, &obj.key).await {
 			Ok((deleted_version, delete_marker_version)) => {
 				if cmd.quiet {
 					continue;
